@@ -12,7 +12,7 @@
  *
  */
 #include "iemmatrix.h"
-void iemmatrix_sources_setup(void);
+#include "iemmatrix_sources.h"
 
 #ifndef BUILD_DATE
 # define BUILD_DATE "" __DATE__ " : " __TIME__
@@ -24,7 +24,7 @@ void iemmatrix_setup()
   post("");
   post("iemmatrix "VERSION);
   post("\tobjects for manipulating 2d-matrices");
-  post("\t(c) 2001-2015 iem");
+  post("\t(c) 2001-2024 iem");
   post("\t\tIOhannes m zmölnig");
   post("\t\tThomas Musil");
   post("\t\tFranz Zotter");
